package lsgui;

import com.sun.jna.Native;
import com.sun.jna.NativeLibrary;
import java.awt.Component;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;
import javax.swing.JFileChooser;
import javax.swing.UIManager;
import javax.swing.filechooser.FileNameExtensionFilter;
import lsgui.gui.MainFrame;
import lsgui.jna.LaterationShooterLibrary;

public class Main {

    public static String[] algorithms;
    public static String[] errorModels;
    public static String[] outputVariants;
    public static volatile boolean operationPending;
    
    private static MainFrame gui;
    private static Properties properties;
    private static final String APP_DIR = "lsgui";
    private static final String PROPERTY_FILENAME = "lsgui.xml";
    
    public static final String PROPERTY_ANCHOR_OPEN_PATH = "pathOpenAnchorFile";
    public static final String PROPERTY_ANCHOR_SAVE_PATH = "pathSaveAnchorFile";
    public static final String PROPERTY_TAB_SAVE_PATH = "pathSaveTabImgFile";
    public static final String PROPERTY_EM_DATA_SAVE_PATH = "pathSaveEMDataFile";
    
    public static LaterationShooterLibrary INSTANCE;

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        
        // Create application directory
        OSSupport.createUserHomeDirectory(APP_DIR);

        loadProperties();
        
        // Try to set the LookAndFeel
        try {
            String laf = UIManager.getSystemLookAndFeelClassName();
            // Try to replace Metal with Nimbus
            if ("javax.swing.plaf.metal.MetalLookAndFeel".equals(laf)) {
                laf = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";
            }
            UIManager.setLookAndFeel(laf);
        } catch (Exception e) {
            System.err.println(e.getLocalizedMessage());
        }
        
        // Add some extra paths to load libraries.
        // JNA looks per default (if nothing specified)
        // into: /usr/lib32, /usr/lib and /lib
        System.setProperty("jna.library.path", ".");
        NativeLibrary.addSearchPath(LaterationShooterLibrary.LIBRARY_NAME,
                "/usr/local/lib");
        
        try {
            INSTANCE = (LaterationShooterLibrary) Native.loadLibrary(
                LaterationShooterLibrary.LIBRARY_NAME,
                LaterationShooterLibrary.class);
            algorithms = INSTANCE.get_algorithms();
            errorModels = INSTANCE.get_error_models();
            outputVariants = INSTANCE.get_result_view_names();
            int checkVariants = INSTANCE.get_number_of_result_views();
            if (checkVariants != outputVariants.length) {
                libraryError("Library error detected, disabling library!");
            }
        } catch (UnsatisfiedLinkError ex) {
            libraryError(ex.toString());
        }
        
        gui = new MainFrame();
        
        try {
            gui.setBounds(Integer.parseInt(
                    properties.getProperty("applicationX", "0")),
                    Integer.parseInt(
                    properties.getProperty("applicationY", "0")),
                    Integer.parseInt(
                    properties.getProperty("applicationWidth", "800")),
                    Integer.parseInt(
                    properties.getProperty("applicationHeight", "600")));
            gui.setExtendedState(Integer.parseInt(
                    properties.getProperty("applicationExtendedState", "0")));
        } catch (NumberFormatException e) {}
        
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                gui.setVisible(true);
            }
        });
    }
    
    private static void libraryError(String error) {
        INSTANCE = null;
        algorithms = new String[] {"-"};
        errorModels = new String[] {"-"};
        outputVariants = null;
        System.err.println(error);
    }
    
    public static void exit() {
        try {
            // Save main window position and state
            properties.setProperty("applicationX",
                    new Integer(gui.getX()).toString());
            properties.setProperty("applicationY",
                    new Integer(gui.getY()).toString());
            properties.setProperty("applicationWidth",
                    new Integer(gui.getWidth()).toString());
            properties.setProperty("applicationHeight",
                    new Integer(gui.getHeight()).toString());
            properties.setProperty("applicationExtendedState",
                    new Integer(gui.getExtendedState()).toString());
            // Save properties to this location
            properties.storeToXML(new FileOutputStream(new File(
                    OSSupport.getUserHomeDirectory(APP_DIR)
                    + PROPERTY_FILENAME)), null);
        } catch (FileNotFoundException ex) {}
          catch (IOException ex) {}
        System.exit(0);
    }
    
    private static void loadProperties() {
        try {
            properties = new Properties();
            properties.loadFromXML(new FileInputStream(new File(
                    OSSupport.getUserHomeDirectory(APP_DIR)
                    + PROPERTY_FILENAME)));
        } catch (FileNotFoundException ex) {}
          catch (IOException ex) {}
    }

    public static File showSaveFileDialog(Component parent, String title) {
        return showSaveFileDialog(parent, title, null, null);
    }
    
    public static File showSaveFileDialog(Component parent, String title,
            String initialFilename, String pathProperty) {
        String saveFilePath = properties.getProperty(pathProperty);
        JFileChooser jfc = new JFileChooser(saveFilePath);
        jfc.setDialogTitle(title);
        if (initialFilename != null) {
            jfc.setSelectedFile(new File(initialFilename));
        }
        jfc.setMultiSelectionEnabled(false);
        if (jfc.showSaveDialog(parent) == JFileChooser.APPROVE_OPTION) {
            File f = jfc.getSelectedFile();
            try {
                if (pathProperty != null) {
                    saveFilePath = f.getParentFile() != null ?
                        f.getParentFile().getCanonicalPath() :
                        f.getCanonicalPath();
                    properties.setProperty(pathProperty, saveFilePath);
                }
            } catch (IOException e) {}
            return f;
        }
        return null;
    }
    
    public static File showOpenFileDialog(Component parent, String title,
            String[] extFilters, String[][] extFiltersExtensions) {
        return showOpenFileDialog(parent, title, extFilters, extFiltersExtensions, null);
    }
    
    public static File showOpenFileDialog(Component parent, String title,
            String[] extFilters, String[][] extFiltersExtensions, String pathProperty) {
        String openFilePath = properties.getProperty(pathProperty);
        JFileChooser chooser = new JFileChooser(openFilePath);
        chooser.setDialogTitle(title);
        chooser.setMultiSelectionEnabled(false);
        for (int i = extFilters.length - 1; i >= 0; i--) {
            FileNameExtensionFilter filter = new FileNameExtensionFilter(
                    extFilters[i], extFiltersExtensions[i]);
            chooser.addChoosableFileFilter(filter);
        }
        int returnVal = chooser.showOpenDialog(parent);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            java.io.File selected = chooser.getSelectedFile();
            try {
                if (pathProperty != null) {
                    openFilePath = selected.getParentFile() != null ?
                        selected.getParentFile().getCanonicalPath() :
                        selected.getCanonicalPath();
                    properties.setProperty(pathProperty, openFilePath);
                }
            } catch (IOException e) {}
            return selected;
        }
        return null;
    }

    public static boolean saveByteArray(byte[] b, java.io.File file) {
        java.io.FileOutputStream fos = null;
        try {
            fos = new java.io.FileOutputStream(file);
            fos.write(b);
            return true;
        } catch (java.io.IOException e) {
            return false;
        } finally {
            try { fos.close(); } catch (Exception e) {}
        }
    }
    
}
