package lsgui.gui;

import javax.swing.DefaultListModel;
import javax.swing.JTabbedPane;
import lsgui.util.LaterationImage;

public class CompareDialog extends javax.swing.JDialog {

    private JTabbedPane pane;
    private boolean dialogResult;
    private int sourceIndex;
    private LaterationImage sourceImage;
    private int targetIndex;
    private LaterationImage targetImage;
    
    private class ListEntry {
        public int tabIndex;
        public String name;
        public ListEntry(int tabIndex, String name) {
            this.tabIndex = tabIndex;
            this.name = name;
        }
        @Override
        public String toString() {
            return name;
        }
    }
    
    public CompareDialog(java.awt.Frame parent, boolean modal) throws Exception {
        super(parent, modal);
        this.pane = ((MainFrame) parent).getTabbedPane();
        initComponents();
        initData();
    }

    public LaterationImage getFirstImage() {
        return sourceImage;
    }
    
    public int getFirstImageDataIndex() {
        return sourceIndex;
    }
    
    public LaterationImage getSecondImage() {
        return targetImage;
    }
    
    public int getSecondImageDataIndex() {
        return targetIndex;
    }

    public boolean getDialogResult() {
        return dialogResult;
    }
    
    private void initData() throws Exception {
        int currentIdx = pane.getSelectedIndex();
        TabContent tabC = (TabContent) pane.getSelectedComponent();
        sourceImage = tabC.getScrollableCanvas().getCanvas().getLatImage();
        sourceIndex = sourceImage.getImageIndex();
        if (sourceImage != null && sourceImage.hasImages() && !sourceImage.isCompareImage()) {
            String tab1Title = pane.getTitleAt(currentIdx);
            String s = tab1Title + ", Image " + (sourceImage.getImageIndex() + 1)
                    + "/" + sourceImage.getImageCount() + ": "
                    + sourceImage.getLaterationImageData().getTag();
            lSource.setText("<html>" + s + "</html>");
            
            // build list with all tabs of tabbed pane
            DefaultListModel model = new DefaultListModel();
            for (int i = 0; i < pane.getTabCount(); i++) {
                tabC = (TabContent) pane.getComponentAt(i);
                LaterationImage tmpImg = tabC.getScrollableCanvas().getCanvas().getLatImage();
                if (tmpImg != null && tmpImg != sourceImage && tmpImg.hasImages()
                        && !tmpImg.isCompareImage()
                        && sourceImage.getEngine() == tmpImg.getEngine()
                        && sourceImage.getImageWidth() == tmpImg.getImageWidth()
                        && sourceImage.getImageHeight() == tmpImg.getImageHeight()) {
                    ListEntry entry = new ListEntry(i, pane.getTitleAt(i));
                    model.addElement(entry);
                }
            }
            list1.setModel(model);
        } else {
            throw new Exception("illegal image for compare");
        }
    }
    
    private void updateList2Data(int idx) {
        DefaultListModel model = new DefaultListModel();
        TabContent tabC = (TabContent) pane.getComponentAt(idx);
        targetImage = tabC.getScrollableCanvas().getCanvas().getLatImage();
        for (int i = 0; i < targetImage.getImageCount(); i++) {
            String s = "Image "  + (i + 1) + "/" + targetImage.getImageCount() + ": "
                    + targetImage.getLaterationImageData(i).getTag();
            model.addElement(s);
        }
        list2.setModel(model);
    }
    
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel2 = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        pSource = new javax.swing.JPanel();
        lSource = new javax.swing.JLabel();
        pTarget = new javax.swing.JPanel();
        jPanel5 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        list1 = new javax.swing.JList();
        jPanel4 = new javax.swing.JPanel();
        jScrollPane2 = new javax.swing.JScrollPane();
        list2 = new javax.swing.JList();
        pBottom = new javax.swing.JPanel();
        bOk = new javax.swing.JButton();
        bCancel = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Configure Compare");
        setResizable(false);

        jPanel2.setBorder(javax.swing.BorderFactory.createEmptyBorder(10, 10, 1, 10));
        jPanel2.setLayout(new java.awt.BorderLayout());

        jPanel1.setLayout(new javax.swing.BoxLayout(jPanel1, javax.swing.BoxLayout.PAGE_AXIS));

        pSource.setBorder(javax.swing.BorderFactory.createTitledBorder("First image:"));
        pSource.setLayout(new java.awt.BorderLayout());

        lSource.setText("jLabel1");
        pSource.add(lSource, java.awt.BorderLayout.CENTER);

        jPanel1.add(pSource);

        pTarget.setBorder(javax.swing.BorderFactory.createTitledBorder("Image to compare with:"));
        pTarget.setLayout(new java.awt.BorderLayout());

        jPanel5.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 5, 1, 5));
        jPanel5.setLayout(new javax.swing.BoxLayout(jPanel5, javax.swing.BoxLayout.LINE_AXIS));

        jPanel3.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 5));
        jPanel3.setLayout(new java.awt.BorderLayout());

        list1.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        list1.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                list1ValueChanged(evt);
            }
        });
        jScrollPane1.setViewportView(list1);

        jPanel3.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jPanel5.add(jPanel3);

        jPanel4.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 5, 0, 0));
        jPanel4.setLayout(new java.awt.BorderLayout());

        list2.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        list2.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                list2MouseClicked(evt);
            }
        });
        list2.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                list2ValueChanged(evt);
            }
        });
        jScrollPane2.setViewportView(list2);

        jPanel4.add(jScrollPane2, java.awt.BorderLayout.CENTER);

        jPanel5.add(jPanel4);

        pTarget.add(jPanel5, java.awt.BorderLayout.CENTER);

        jPanel1.add(pTarget);

        jPanel2.add(jPanel1, java.awt.BorderLayout.CENTER);

        getContentPane().add(jPanel2, java.awt.BorderLayout.CENTER);

        bOk.setText("Ok");
        bOk.setEnabled(false);
        bOk.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bOkActionPerformed(evt);
            }
        });
        pBottom.add(bOk);

        bCancel.setText("Cancel");
        bCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bCancelActionPerformed(evt);
            }
        });
        pBottom.add(bCancel);

        getContentPane().add(pBottom, java.awt.BorderLayout.SOUTH);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-451)/2, (screenSize.height-447)/2, 451, 447);
    }// </editor-fold>//GEN-END:initComponents

    private void bCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bCancelActionPerformed
        dispose();
    }//GEN-LAST:event_bCancelActionPerformed

    private void list1ValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_list1ValueChanged
        if (!evt.getValueIsAdjusting()) {
            int idx = list1.getSelectedIndex();
            if (idx != -1) {
                ListEntry entry = (ListEntry) list1.getModel().getElementAt(idx);
                updateList2Data(entry.tabIndex);
            }
        }
    }//GEN-LAST:event_list1ValueChanged

    private void list2ValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_list2ValueChanged
        if (!evt.getValueIsAdjusting()) {
            bOk.setEnabled(list2.getSelectedIndex() != -1);
        }
    }//GEN-LAST:event_list2ValueChanged

    private void bOkActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bOkActionPerformed
        if (targetImage != null) {
            dialogResult = true;
            targetIndex = list2.getSelectedIndex();
            dispose();
        }
    }//GEN-LAST:event_bOkActionPerformed

    private void list2MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_list2MouseClicked
        if (evt.getClickCount() == 2) {
            if (bOk.isEnabled()) {
                bOk.doClick();
            }
        }
    }//GEN-LAST:event_list2MouseClicked

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bCancel;
    private javax.swing.JButton bOk;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JLabel lSource;
    private javax.swing.JList list1;
    private javax.swing.JList list2;
    private javax.swing.JPanel pBottom;
    private javax.swing.JPanel pSource;
    private javax.swing.JPanel pTarget;
    // End of variables declaration//GEN-END:variables
}
