package lsgui.gui;

import lsgui.Main;
import lsgui.util.LaterationImage;
import lsgui.util.LaterationImageListener;

public class TabContent extends javax.swing.JPanel implements LaterationImageListener {

    private MainFrame gui;
    private ScrollableCanvas scrollableCanvas;
    
    public TabContent(MainFrame gui, ScrollableCanvas scrollableCanvas) {
        this.gui = gui;
        this.scrollableCanvas = scrollableCanvas;
        init0();
    }

    private void init0() {
        this.scrollableCanvas.getCanvas().addLaterationImageListener(this);
        initComponents();
        add(scrollableCanvas, java.awt.BorderLayout.CENTER);
        setRuntime(null);
        imageChanged();
    }
    
    public ScrollableCanvas getScrollableCanvas() {
        return scrollableCanvas;
    }
    
    public void setRuntime(String s) {
        lRuntime.setText(s);
    }
    
    public void setControlsEnabled(boolean enabled) {
        Canvas canvas = scrollableCanvas.getCanvas();
        LaterationImage image = canvas.getLatImage();
        if (image != null && !image.isCompareImage()) {
            bCompareImage.setEnabled(enabled);
        }
    }

    public void imageChanged() {
        Canvas canvas = scrollableCanvas.getCanvas();
        LaterationImage image = canvas.getLatImage();
        updateState(image, canvas);
    }
    
    private void updateState(LaterationImage image, Canvas canvas) {
        bPrevImage.setEnabled(image != null);
        bNextImage.setEnabled(image != null);
        bFirstImage.setEnabled(image != null);
        bLastImage.setEnabled(image != null);
        bChart.setEnabled(image != null && !image.isCompareImage());
        bCompareImage.setEnabled(image != null && !image.isCompareImage() && !Main.operationPending);
        if (image == null) {
            lImageCount.setText(null);
            lImageType.setText(null);
        } else {
            lImageCount.setText("  " + image.getImageWidth() + " x " + image.getImageHeight()
                    + "px   Image " + (image.getImageIndex() + 1) + "/"
                    + image.getImageCount() + ": ");
            lImageType.setText(image.getLaterationImageData().getTag());
        }
        canvas.repaintRequest();
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        tbTop = new javax.swing.JToolBar();
        bChart = new javax.swing.JToggleButton();
        bFirstImage = new javax.swing.JButton();
        bPrevImage = new javax.swing.JButton();
        bNextImage = new javax.swing.JButton();
        bLastImage = new javax.swing.JButton();
        jSeparator1 = new javax.swing.JToolBar.Separator();
        bCompareImage = new javax.swing.JButton();
        lImageCount = new javax.swing.JLabel();
        lImageType = new javax.swing.JLabel();
        pBottom = new javax.swing.JPanel();
        lRuntime = new javax.swing.JLabel();

        setLayout(new java.awt.BorderLayout());

        tbTop.setFloatable(false);
        tbTop.setRollover(true);

        bChart.setIcon(new javax.swing.ImageIcon(getClass().getResource("/lsgui/images/chart_bar.png"))); // NOI18N
        bChart.setToolTipText("Toggle statistics");
        bChart.setFocusable(false);
        bChart.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bChart.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bChart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bChartActionPerformed(evt);
            }
        });
        tbTop.add(bChart);

        bFirstImage.setIcon(new javax.swing.ImageIcon(getClass().getResource("/lsgui/images/resultset_first.png"))); // NOI18N
        bFirstImage.setToolTipText("First image");
        bFirstImage.setFocusable(false);
        bFirstImage.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bFirstImage.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bFirstImage.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bFirstImageActionPerformed(evt);
            }
        });
        tbTop.add(bFirstImage);

        bPrevImage.setIcon(new javax.swing.ImageIcon(getClass().getResource("/lsgui/images/resultset_previous.png"))); // NOI18N
        bPrevImage.setToolTipText("Previous image");
        bPrevImage.setFocusable(false);
        bPrevImage.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bPrevImage.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bPrevImage.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bPrevImageActionPerformed(evt);
            }
        });
        tbTop.add(bPrevImage);

        bNextImage.setIcon(new javax.swing.ImageIcon(getClass().getResource("/lsgui/images/resultset_next.png"))); // NOI18N
        bNextImage.setToolTipText("Next image");
        bNextImage.setFocusable(false);
        bNextImage.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bNextImage.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bNextImage.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bNextImageActionPerformed(evt);
            }
        });
        tbTop.add(bNextImage);

        bLastImage.setIcon(new javax.swing.ImageIcon(getClass().getResource("/lsgui/images/resultset_last.png"))); // NOI18N
        bLastImage.setToolTipText("Last image");
        bLastImage.setFocusable(false);
        bLastImage.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bLastImage.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bLastImage.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bLastImageActionPerformed(evt);
            }
        });
        tbTop.add(bLastImage);
        tbTop.add(jSeparator1);

        bCompareImage.setIcon(new javax.swing.ImageIcon(getClass().getResource("/lsgui/images/photos.png"))); // NOI18N
        bCompareImage.setToolTipText("Compare images");
        bCompareImage.setFocusable(false);
        bCompareImage.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        bCompareImage.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        bCompareImage.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bCompareImageActionPerformed(evt);
            }
        });
        tbTop.add(bCompareImage);

        lImageCount.setText("jLabel1");
        tbTop.add(lImageCount);

        lImageType.setText("jLabel1");
        tbTop.add(lImageType);

        add(tbTop, java.awt.BorderLayout.NORTH);

        pBottom.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        lRuntime.setText("lTime");
        pBottom.add(lRuntime);

        add(pBottom, java.awt.BorderLayout.SOUTH);
    }// </editor-fold>//GEN-END:initComponents

    private void bPrevImageActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bPrevImageActionPerformed
        Canvas canvas = scrollableCanvas.getCanvas();
        LaterationImage image = canvas.getLatImage();
        if (image != null && image.hasImages()) {
            image.previousImage();
            updateState(image, canvas);
        }
    }//GEN-LAST:event_bPrevImageActionPerformed

    private void bNextImageActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bNextImageActionPerformed
        Canvas canvas = scrollableCanvas.getCanvas();
        LaterationImage image = canvas.getLatImage();
        if (image != null && image.hasImages()) {
            image.nextImage();
            updateState(image, canvas);
        }
    }//GEN-LAST:event_bNextImageActionPerformed

    private void bChartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bChartActionPerformed
        Canvas canvas = scrollableCanvas.getCanvas();
        canvas.setStatisticsEnabled(bChart.isSelected());
    }//GEN-LAST:event_bChartActionPerformed

    private void bFirstImageActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bFirstImageActionPerformed
        Canvas canvas = scrollableCanvas.getCanvas();
        LaterationImage image = canvas.getLatImage();
        if (image != null && image.hasImages()) {
            image.firstImage();
            updateState(image, canvas);
        }
    }//GEN-LAST:event_bFirstImageActionPerformed

    private void bLastImageActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bLastImageActionPerformed
        Canvas canvas = scrollableCanvas.getCanvas();
        LaterationImage image = canvas.getLatImage();
        if (image != null && image.hasImages()) {
            image.lastImage();
            updateState(image, canvas);
        }
    }//GEN-LAST:event_bLastImageActionPerformed

    private void bCompareImageActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bCompareImageActionPerformed
        try {
            final CompareDialog dialog = new CompareDialog(gui, true);
            java.awt.EventQueue.invokeLater(new Runnable() {
                public void run() {
                    dialog.setVisible(true);
                    if (dialog.getDialogResult()) {
                        gui.compare(dialog.getFirstImage(),
                                dialog.getFirstImageDataIndex(),
                                dialog.getSecondImage(),
                                dialog.getSecondImageDataIndex());
                    }
                }
            });
        } catch (Exception ex) {}
    }//GEN-LAST:event_bCompareImageActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JToggleButton bChart;
    private javax.swing.JButton bCompareImage;
    private javax.swing.JButton bFirstImage;
    private javax.swing.JButton bLastImage;
    private javax.swing.JButton bNextImage;
    private javax.swing.JButton bPrevImage;
    private javax.swing.JToolBar.Separator jSeparator1;
    private javax.swing.JLabel lImageCount;
    private javax.swing.JLabel lImageType;
    private javax.swing.JLabel lRuntime;
    private javax.swing.JPanel pBottom;
    private javax.swing.JToolBar tbTop;
    // End of variables declaration//GEN-END:variables

}
